package ProjectElectricLocomotive.src.MovementStrategy;

public abstract class AbstractStrategy
{
    private IMoveableObject _moveableObject = null;
    private StrategyStatus _state = StrategyStatus.NotInit;
    protected int fieldWidth;
    protected int fieldHeight;
    protected StrategyStatus state;

    protected ObjectParameters getObjectParameters() {
        return (_moveableObject != null) ? _moveableObject.getObjectPosition() : null;
    }

    protected Integer GetStep() {
        if (state != StrategyStatus.InProgress) {
            return null;
        }
        return (_moveableObject != null) ? _moveableObject.getStep() : null;
    }

    public StrategyStatus getStatus() {
        return state;
    }

    public void SetData(IMoveableObject moveableObject, int width, int height) {
        if (moveableObject == null) {
            state = StrategyStatus.NotInit;
            return;
        }
        state = StrategyStatus.InProgress;
        this._moveableObject = moveableObject;
        fieldWidth = width;
        fieldHeight = height;
    }

    public void MakeStep() {
        if (state != StrategyStatus.InProgress) {
            return;
        }
        if (isTargetDestination()) {
            state = StrategyStatus.Finish;
            return;
        }
        moveToTarget();
    }

    protected boolean MoveLeft() {
        return moveTo(MovementDirection.Left);
    }

    protected boolean MoveRight() {
        return moveTo(MovementDirection.Right);
    }

    protected boolean MoveUp() {
        return moveTo(MovementDirection.Up);
    }

    protected boolean MoveDown() {
        return moveTo(MovementDirection.Down);
    }

    private boolean moveTo(MovementDirection movementDirection) {
        if (state != StrategyStatus.InProgress) {
            return false;
        }
        return (_moveableObject != null) && _moveableObject.tryMoveObject(movementDirection);
    }

    protected abstract void moveToTarget();

    protected abstract boolean isTargetDestination();
}